# -*- coding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2010, 2degrees Limited <gustavonarea@2degreesnetwork.com>.
# All Rights Reserved.
#
# This file is part of twod.wsgi <http://bitbucket.org/2degrees/twod.wsgi/>,
# which is subject to the provisions of the BSD at
# <http://dev.2degreesnetwork.com/p/2degrees-license.html>. A copy of the
# license should accompany this distribution. THIS SOFTWARE IS PROVIDED "AS IS"
# AND ANY AND ALL EXPRESS OR IMPLIED WARRANTIES ARE DISCLAIMED, INCLUDING, BUT
# NOT LIMITED TO, THE IMPLIED WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST
# INFRINGEMENT, AND FITNESS FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""
Miscellaneous PasteDeploy Application Factories.

"""
from os import path

from paste.urlmap import URLMap
from paste.urlparser import StaticURLParser
from django import __file__ as django_init


__all__ = ("make_full_django_app", "add_media_to_app")


_DJANGO_ROOT = path.dirname(django_init)


def make_full_django_app(loader, global_conf, **local_conf):
    """
    Return a WSGI application made up of the Django application, its media and
    the Django Admin media.
    
    This is a PasteDeploy Composite Application Factory.
    
    """
    django_app = loader.get_app(local_conf['django_app'], global_conf=global_conf)
    return add_media_to_app(django_app)


def add_media_to_app(django_app):
    """
    Return a WSGI application made up of the Django application, its media and
    the Django Admin media.
    
    """
    app = URLMap()
    app['/'] = django_app
    
    # The Django App has been loaded, so it's now safe to access the settings:
    from django.conf import settings
    
    # Setting up the Admin media:
    admin_media = path.join(_DJANGO_ROOT, "contrib", "admin", "media")
    app[settings.ADMIN_MEDIA_PREFIX] = StaticURLParser(admin_media)
    
    # Setting up the media for the Django application:
    app[settings.MEDIA_URL] = StaticURLParser(settings.MEDIA_ROOT)
    
    return app
    




# -*- coding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2010, 2degrees Limited <gustavonarea@2degreesnetwork.com>.
# All Rights Reserved.
#
# This file is part of twod.wsgi <http://bitbucket.org/2degrees/twod.wsgi/>,
# which is subject to the provisions of the BSD at
# <http://dev.2degreesnetwork.com/p/2degrees-license.html>. A copy of the
# license should accompany this distribution. THIS SOFTWARE IS PROVIDED "AS IS"
# AND ANY AND ALL EXPRESS OR IMPLIED WARRANTIES ARE DISCLAIMED, INCLUDING, BUT
# NOT LIMITED TO, THE IMPLIED WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST
# INFRINGEMENT, AND FITNESS FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""
Utilities to set up Django applications, both in Web and CLI environments.

"""
import os
from logging import getLogger

from paste.deploy.loadwsgi import appconfig
from paste.deploy.converters import asbool, asint, aslist

from twod.wsgi.handler import DjangoApplication


__all__ = ("wsgify_django", )

_LOGGER = getLogger(__name__)


def wsgify_django(global_config, **local_conf):
    """
    Load the Django application for use in a WSGI server.
    
    :raises ImportError: If the Django settings module cannot be imported.
    :raises ValueError: If ``local_conf`` contains a Django setting which is
        not supported.
    :raises ValueError: If the ``django_settings_module`` directive is not set.
    :raises ValueError: If Django's ``DEBUG`` is set instead of Paste's
        ``debug``.
    :return: The Django application as a WSGI application.
    :rtype: :class:`~twod.wsgi.handler.DjangoApplication`
    
    """
    _set_up_settings(global_config, local_conf)
    return DjangoApplication()


def _set_up_settings(global_conf, local_conf):
    """
    Add the PasteDeploy options to the DJANGO_SETTINGS_MODULE module.
    
    """
    django_settings_module = global_conf.get("django_settings_module")
    if not django_settings_module:
        raise ValueError('The "django_settings_module" directive is not set')
    
    os.environ['DJANGO_SETTINGS_MODULE'] = django_settings_module
    
    # Attaching the variables to the settings module, at least those which had
    # not been defined.
    # We need the module name for __import__ to work properly:
    # http://stackoverflow.com/questions/211100/pythons-import-doesnt-work-as-expected
    module = django_settings_module.split(".")[-1]
    settings_module = __import__(django_settings_module, fromlist=[module])
    
    if hasattr(settings_module, "DEBUG"):
        raise ValueError('Module %s must not define "DEBUG". It must be set '
                         'in the PasteDesploy configuration file as "debug".' %
                         django_settings_module)
    
    options = _convert_options(global_conf, local_conf)
    
    for (setting_name, setting_value) in options.items():
        if not hasattr(settings_module, setting_name):
            # The name is not used; let's set it:
            setattr(settings_module, setting_name, setting_value)
        elif isinstance(getattr(settings_module, setting_name), tuple):
            # The name is already used by a list; let's extended it:
            new_tuple = getattr(settings_module, setting_name) + setting_value
            setattr(settings_module, setting_name, new_tuple)
        else:
            # The name is already used and it's not a list; let's warn the user:
            _LOGGER.warn('"%s" will not be overridden in %s', setting_name,
                         django_settings_module)


#{ Type casting


# Official Django settings, excerpted from
# http://docs.djangoproject.com/en/dev/ref/settings/
_DJANGO_BOOLEANS = frozenset([
    "APPEND_SLASH",
    "DEBUG",
    "DEBUG_PROPAGATE_EXCEPTIONS",
    "EMAIL_USE_TLS",
    "FORCE_SCRIPT_NAME",
    "PREPEND_WWW",
    "SEND_BROKEN_LINK_EMAILS",
    "SESSION_EXPIRE_AT_BROWSER_CLOSE",
    "SESSION_SAVE_EVERY_REQUEST",
    "TEMPLATE_DEBUG",
    "USE_ETAGS",
    "USE_I18N",
    "USE_L10N",
    "USE_THOUSAND_SEPARATOR",
    ])

_DJANGO_INTEGERS = frozenset([
    "CACHE_MIDDLEWARE_SECONDS",
    "EMAIL_PORT",
    "FILE_UPLOAD_MAX_MEMORY_SIZE",
    "FIRST_DAY_OF_WEEK",
    "MESSAGE_LEVEL",
    "NUMBER_GROUPING",
    "SESSION_COOKIE_AGE",
    "SESSION_COOKIE_SECURE",
    "SITE_ID",
    ])

_DJANGO_TUPLES = frozenset([
    "ADMIN_FOR",
    "AUTHENTICATION_BACKENDS",
    "ALLOWED_INCLUDE_ROOTS",
    "DATABASE_ROUTERS",
    "DATE_INPUT_FORMATS",
    "DATETIME_INPUT_FORMATS",
    "DISALLOWED_USER_AGENTS",
    "FILE_UPLOAD_HANDLERS",
    "FIXTURE_DIRS",
    "IGNORABLE_404_ENDS",
    "IGNORABLE_404_STARTS",
    "INSTALLED_APPS",
    "INTERNAL_IPS",
    "LOCALE_PATHS",
    "MIDDLEWARE_CLASSES",
    "PROFANITIES_LIST",
    "TEMPLATE_CONTEXT_PROCESSORS",
    "TEMPLATE_DIRS",
    "TEMPLATE_LOADERS",
    "TIME_INPUT_FORMATS",
    ])

_DJANGO_NESTED_TUPLES = frozenset([
    "ADMINS",
    "MANAGERS",
    ])

_DJANGO_DICTIONARIES = frozenset(["DATABASE_OPTIONS"])

_DJANGO_NONE_IF_EMPTY_SETTINGS = frozenset([
    "CSRF_COOKIE_DOMAIN",
    "FILE_UPLOAD_TEMP_DIR",
    "FORMAT_MODULE_PATH",
    "SESSION_COOKIE_DOMAIN",
    "SESSION_FILE_PATH",
    "STATIC_URL",
    ])

# TODO: The following settings should be supported:
_DJANGO_UNSUPPORTED_SETTINGS = frozenset([
    "FILE_UPLOAD_PERMISSIONS",
    "LANGUAGES",
    "MESSAGE_TAGS",
    "SERIALIZATION_MODULES",
    ])


def _convert_options(global_conf, local_conf):
    """
    Build the final options based on PasteDeploy's ``global_conf`` and
    ``local_conf``.
    
    """
    # First of all, let's make sure Django will use Paste's "debug" value:
    if "DEBUG" in global_conf or "DEBUG" in local_conf:
        raise ValueError("Do not set Django's DEBUG in the configuration file; "
                         "use Paste's 'debug' instead")
    if "debug" not in global_conf:
        raise ValueError("Paste's 'debug' option must be set in the "
                         "configuration file")
    
    local_conf['DEBUG'] = global_conf['debug']
    
    # Now it's safe to move on with the type casting:
    
    custom_booleans = aslist(global_conf.get("twod.booleans", ""))
    custom_integers = aslist(global_conf.get("twod.integers", ""))
    custom_tuples = aslist(global_conf.get("twod.tuples", ""))
    custom_nested_tuples = aslist(global_conf.get("twod.nested_tuples", ""))
    custom_dictionaries = aslist(global_conf.get("twod.dictionaries", ""))
    custom_none_if_empty_settings = aslist(
        global_conf.get("twod.none_if_empty_settings", "")
        )
    
    booleans = _DJANGO_BOOLEANS | frozenset(custom_booleans)
    integers = _DJANGO_INTEGERS | frozenset(custom_integers)
    tuples = _DJANGO_TUPLES | frozenset(custom_tuples)
    nested_tuples = _DJANGO_NESTED_TUPLES | frozenset(custom_nested_tuples)
    dictionaries = _DJANGO_DICTIONARIES | frozenset(custom_dictionaries)
    none_if_empty_settings = (_DJANGO_NONE_IF_EMPTY_SETTINGS | 
                              frozenset(custom_none_if_empty_settings))
    
    options = {}
    for (option_name, option_value) in local_conf.items():
        if option_name in booleans:
            options[option_name] = asbool(option_value)
        elif option_name in integers:
            options[option_name] = asint(option_value)
        elif option_name in tuples:
            options[option_name] = tuple(aslist(option_value))
        elif option_name in nested_tuples:
            nested_tuple = tuple(
                tuple((val.strip()) for val in tuple_.strip().split(";") if val)
                for tuple_ in option_value.strip().splitlines()
                )
            options[option_name] = nested_tuple
        elif option_name in dictionaries:
            lines = option_value.strip().splitlines()
            items = [option.strip().split("=", 1) for option in lines]
            dictionary = dict(
                (key.strip(), value.strip()) for (key, value) in items
                )
            options[option_name] = dictionary
        elif option_name in none_if_empty_settings:
            value = option_value.strip()
            if value:
                value_to_use = value
            else:
                value_to_use = None
            options[option_name] = value_to_use
        elif option_name in _DJANGO_UNSUPPORTED_SETTINGS:
            raise ValueError("Django setting %s is not (yet) supported; "
                             "you have to define it in your options module." %
                             option_name)
        else:
            # Store the option as an string:
            options[option_name] = option_value
    
    # We should not import a module with "__file__" as a global variable:
    options['paste_configuration_file'] = global_conf.get("__file__")
    
    return options


#}



# -*- coding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2010, 2degrees Limited <gustavonarea@2degreesnetwork.com>.
# All Rights Reserved.
#
# This file is part of twod.wsgi <http://bitbucket.org/2degrees/twod.wsgi/>,
# which is subject to the provisions of the BSD at
# <http://dev.2degreesnetwork.com/p/2degrees-license.html>. A copy of the
# license should accompany this distribution. THIS SOFTWARE IS PROVIDED "AS IS"
# AND ANY AND ALL EXPRESS OR IMPLIED WARRANTIES ARE DISCLAIMED, INCLUDING, BUT
# NOT LIMITED TO, THE IMPLIED WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST
# INFRINGEMENT, AND FITNESS FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""
Enhanced WSGI support for Django applications.

"""

# Importing elements that should be available from this namespace:
from twod.wsgi.handler import DjangoApplication, TwodResponse
from twod.wsgi.middleware import RoutingArgsMiddleware
from twod.wsgi.embedded_wsgi import call_wsgi_app, make_wsgi_view
from twod.wsgi.appsetup import wsgify_django

__all__ = ("DjangoApplication", "TwodResponse", "RoutingArgsMiddleware",
           "call_wsgi_app", "make_wsgi_view", "wsgify_django")



# -*- coding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2010, 2degrees Limited <gustavonarea@2degreesnetwork.com>.
# All Rights Reserved.
#
# This file is part of twod.wsgi <http://bitbucket.org/2degrees/twod.wsgi/>,
# which is subject to the provisions of the BSD at
# <http://dev.2degreesnetwork.com/p/2degrees-license.html>. A copy of the
# license should accompany this distribution. THIS SOFTWARE IS PROVIDED "AS IS"
# AND ANY AND ALL EXPRESS OR IMPLIED WARRANTIES ARE DISCLAIMED, INCLUDING, BUT
# NOT LIMITED TO, THE IMPLIED WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST
# INFRINGEMENT, AND FITNESS FOR A PARTICULAR PURPOSE.
#
##############################################################################

import os

from ez_setup import use_setuptools
use_setuptools()

from setuptools import setup, find_packages

here = os.path.abspath(os.path.dirname(__file__))
README = open(os.path.join(here, "README.txt")).read()
version = open(os.path.join(here, "VERSION.txt")).readline().rstrip()

setup(name="twod.wsgi",
      version=version,
      description="Enhanced WSGI support for Django applications",
      long_description=README,
      classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Environment :: Web Environment",
        "Framework :: Django",
        "Framework :: Paste",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: BSD License",
        "Natural Language :: English",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 2",
        "Topic :: Internet :: WWW/HTTP",
        "Topic :: Internet :: WWW/HTTP :: WSGI",
        "Topic :: Security",
        ],
      keywords="django wsgi paste webob web",
      author="2degrees Limited",
      author_email="2degrees-floss@googlegroups.com",
      namespace_packages=["twod"],
      url="http://packages.python.org/twod.wsgi/",
      license="BSD (http://dev.2degreesnetwork.com/p/2degrees-license.html)",
      packages=find_packages(exclude=["tests"]),
      py_modules=["django_testing", "django_testing_recipe"],
      zip_safe=False,
      tests_require = [
        "coverage",
        ],
      install_requires=[
        "Django >= 1.1",
        "WebOb >= 0.9.7",
        "Paste >= 1.7.2",
        "PasteDeploy >= 1.3.3",
        "setuptools",
        "nose",
        ],
      extras_require = {
        'buildout': ["zc.recipe.egg >= 1.2.2"],
        },
      test_suite="nose.collector",
      entry_points = """\
        [paste.app_factory]
        main = twod.wsgi.appsetup:wsgify_django
        
        [paste.composite_factory]
        full_django = twod.wsgi.factories:make_full_django_app
        
        [nose.plugins.0.10]
        django-wsgified = django_testing:DjangoWsgifiedPlugin
        
        [zc.buildout]
        nose = django_testing_recipe:DjangoWsgifiedRecipe [buildout]
      """
      )



# -*- coding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2010, 2degrees Limited <gustavonarea@2degreesnetwork.com>.
# All Rights Reserved.
#
# This file is part of twod.wsgi <http://bitbucket.org/2degrees/twod.wsgi/>,
# which is subject to the provisions of the BSD at
# <http://dev.2degreesnetwork.com/p/2degrees-license.html>. A copy of the
# license should accompany this distribution. THIS SOFTWARE IS PROVIDED "AS IS"
# AND ANY AND ALL EXPRESS OR IMPLIED WARRANTIES ARE DISCLAIMED, INCLUDING, BUT
# NOT LIMITED TO, THE IMPLIED WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST
# INFRINGEMENT, AND FITNESS FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""
Nose plugin to run Django applications in a WSGI environment.

This module has no automated tests on purpose. Functional tests would be very
useful.

"""
from nose.plugins import Plugin
from paste.deploy import loadapp


__all__ = ("DjangoWsgifiedPlugin",)


class DjangoWsgifiedPlugin(Plugin):
    """
    Loads the Django application described by the PasteDeploy configuration URL
    in a WSGI environment suitable for testing.
    
    """
    enabled = False
    
    name = "django-wsgified"
    
    enableOpt = "paste_config_uri"
    
    def options(self, parser, env):
        help = (
            "Load the Django application described by the PasteDeploy "
            "configuration URI in a WSGI environment suitable for testing."
        )
        
        parser.add_option(
            "--with-%s" % self.name,
            type="string",
            default="",
            dest=self.enableOpt,
            help=help,
            )
        
        parser.add_option(
            "--no-db",
            action="store_true",
            default=False,
            dest="no_db",
            # We must mention "Django" so people know where the option comes
            # from:
            help="Do not set up a Django test database",
            )
    
    def configure(self, options, conf):
        """Store the URI to the PasteDeploy configuration."""
        super(DjangoWsgifiedPlugin, self).configure(options, conf)
        
        self.paste_config_uri = getattr(options, self.enableOpt)
        self.enabled = bool(self.paste_config_uri)
        self.verbosity = options.verbosity
        self.create_db = not options.no_db
    
    def begin(self):
        loadapp(self.paste_config_uri)
        # It's safe to import from Django at this point. The infamous
        # DJANGO_SETTINGS_MODULE is now set.
        
        # Setting up Django:
        from django.test.utils import setup_test_environment
        setup_test_environment()
        
        # Setting up the database:
        if self.create_db:
            from django.conf import settings
            from django.db import connection
            
            self.db_name = settings.DATABASE_NAME
            connection.creation.create_test_db(self.verbosity, autoclobber=True)
    
    def finalize(self, result=None):
        from django.test.utils import teardown_test_environment
        teardown_test_environment()
        # Tearing down the database:
        if self.create_db:
            from django.db import connection
            connection.creation.destroy_test_db(self.db_name, self.verbosity)



# -*- coding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2010, 2degrees Limited <gustavonarea@2degreesnetwork.com>.
# All Rights Reserved.
#
# This file is part of twod.wsgi <http://bitbucket.org/2degrees/twod.wsgi/>,
# which is subject to the provisions of the BSD at
# <http://dev.2degreesnetwork.com/p/2degrees-license.html>. A copy of the
# license should accompany this distribution. THIS SOFTWARE IS PROVIDED "AS IS"
# AND ANY AND ALL EXPRESS OR IMPLIED WARRANTIES ARE DISCLAIMED, INCLUDING, BUT
# NOT LIMITED TO, THE IMPLIED WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST
# INFRINGEMENT, AND FITNESS FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""
Buildout recipe to use the Nose plugin
:class:`django_testing.DjangoWsgifiedPlugin`.

"""
from zc.buildout import UserError
from zc.recipe.egg import Scripts


class DjangoWsgifiedRecipe(Scripts):
    
    def __init__(self, buildout, name, options):
        config_uri = options.pop("paste_config_uri", None)
        if not config_uri:
            raise UserError("Part [%s] must define the PasteDeploy config URI "
                            "in 'paste_config_uri'" % name)
        
        options['initialization'] = _INITIALIZATION % config_uri
        options['arguments'] = "argv=args"
        options['scripts'] = "nosetests"
        super(DjangoWsgifiedRecipe, self).__init__(buildout, name, options)


_INITIALIZATION = """
from sys import argv
args = [argv[0], "--with-django-wsgified=%s"] + argv[1:]
"""



# -*- coding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2010, 2degrees Limited <gustavonarea@2degreesnetwork.com>.
# All Rights Reserved.
#
# This file is part of twod.wsgi <http://bitbucket.org/2degrees/twod.wsgi/>,
# which is subject to the provisions of the BSD at
# <http://dev.2degreesnetwork.com/p/2degrees-license.html>. A copy of the
# license should accompany this distribution. THIS SOFTWARE IS PROVIDED "AS IS"
# AND ANY AND ALL EXPRESS OR IMPLIED WARRANTIES ARE DISCLAIMED, INCLUDING, BUT
# NOT LIMITED TO, THE IMPLIED WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST
# INFRINGEMENT, AND FITNESS FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""
Django request/response handling a la WSGI.

"""
from webob import Request
from django.core.handlers.wsgi import WSGIRequest, WSGIHandler
from django.http import HttpResponse

__all__ = ("TwodWSGIRequest", "TwodResponse", "DjangoApplication")


_ACTUAL_REASON_HEADER = "X-Actual-Status-Reason"


class TwodWSGIRequest(WSGIRequest, Request):
    """
    Pythonic proxy for the WSGI environment.
    
    This class is the Django request extended by WebOb's request. Where they
    both have the same members, Django's take precedence. For example, ``.GET``
    uses :attr:`django.core.handlers.wsgi.WSGIRequest.GET` instead of
    :attr:`webob.Request.GET`.
    
    To access WebOb's GET and POST dictionaries, you have to use ``.uGET``
    and ``.uPOST`` respectively.
    
    """
    
    # The following are used as attributes by Django's request class instances.
    # If we don't set them class-wide, WebOb is going to put them into its
    # ``webob.adhoc_attrs`` variable. See webob.Request:__setattr__.
    environ = None
    path = None
    method = None
    META = None
    
    def __init__(self, environ):
        Request.__init__(self, environ)
        WSGIRequest.__init__(self, environ)
    
    #{ Handing arguments
    
    uPOST = Request.POST
    uGET = Request.GET
    
    # webob.Request
    @property
    def str_POST(self):
        """
        Return the POST arguments by using WebOb.
        
        """
        # Before returning the POST arguments, we have to restore the content
        # length, which is reset by WebOb:
        original_content_length = self.environ.get("CONTENT_LENGTH")
        try:
            return super(TwodWSGIRequest, self).str_POST
        finally:
            self.environ['CONTENT_LENGTH'] = original_content_length
            # "Resetting" the input so Django will read it:
            self._seek_input()
    
    # django.core.handlers.wsgi.WSGIRequest
    def _load_post_and_files(self):
        """
        Parse the POST arguments and uploaded files by using Django.
        
        """
        try:
            return super(TwodWSGIRequest, self)._load_post_and_files()
        finally:
            # "Resetting" the input so WebOb will read it:
            self._seek_input()
    
    def _seek_input(self):
        if "wsgi.input" in self.environ:
            self.environ['wsgi.input'].seek(0)
    
    #}


class TwodResponse(HttpResponse):
    """
    Django-based response class which keeps the HTTP status reason phrase.
    
    The original implementation in Django simply keeps the status code and
    does not allow developers to use custom HTTP status reason phrase, which
    is explicitly allowed by the HTTP specification:
    http://www.w3.org/Protocols/rfc2616/rfc2616-sec6.html
    
    """
    
    def __init__(self, content="", mimetype=None, status=None, *args, **kwargs):
        if isinstance(status, basestring):
            # The status is given as a real HTTP status header, so we should
            # tell the code and the reason apart for Django:
            (status_code, status_reason) = status.split(" ", 1)
            status_code = int(status_code)
            self.status_reason = status_reason or None
        else:
            # The status has been given the old way supported by Django, simply
            # the status code as an integer:
            status_code = status
            self.status_reason = None
        
        super(TwodResponse, self).__init__(content, mimetype, status_code,
                                           *args, **kwargs)
        
        # Including the actual reason header if necessary:
        if self.status_reason:
            reason_header = "%s %s" % (self.status_code, self.status_reason)
            self._headers[_ACTUAL_REASON_HEADER.lower()] = \
                (_ACTUAL_REASON_HEADER, reason_header)


class DjangoApplication(WSGIHandler):
    """
    Django request handler which uses our enhanced WSGI request class.
    
    """
    request_class = TwodWSGIRequest
    
    def __call__(self, environ, start_response):
        start_response_wrapper = _StartResponseWrapper(start_response)
        return super(DjangoApplication, self).__call__(environ,
                                                       start_response_wrapper)


#{ Internals


class _StartResponseWrapper(object):
    """
    Wrapper for an actual start_response() callable which replaces the
    ``status`` with the actual status reason, if any.
    
    We need to iterate over all the ``response_headers`` until we find the
    actual status reason. If it's not there, we'd have wasted time and
    resources, but there's no other way around this until Django fixes the bug:
    http://code.djangoproject.com/ticket/12747
    
    """
    
    def __init__(self, original_start_response):
        self.original_start_response = original_start_response
    
    def __call__(self, status, response_headers, exc_info=None):
        final_headers = []
        
        for (header_name, header_value) in response_headers:
            if header_name == _ACTUAL_REASON_HEADER:
                status = header_value
            else:
                final_headers.append((header_name, header_value))
        
        return self.original_start_response(status, final_headers)


#}
